/*
//
//  This is a basic example of a speech to text (STT) and text to speech
//  loop. The computer listens for input, and then speaks the recognized
//  result. Uses GOOGLE TTS and STT services.
//  
//  Requires getflourish.com ASR library and an internet connection to
//  google. 
//  
//  Prepared for a workshop at UNTREF, 9/21/2013
//  http://wiki.roberttwomey.com/UNTREF_Speech_Workshop
//  
//  Robert Twomey rtwomey@uw.edu 2013
//
 */


// STT //
import com.getflourish.stt.*;
import ddf.minim.*;
import javax.sound.sampled.*;

// TTS //
//import ddf.minim.*;
// necessary java inputs for processing 2.0
import java.net.*;
import java.io.*;//OutputStream;


// Recognition //
STT stt;
String result;

// Synthesis //
AudioPlayer player;

// Both //
Minim minim;

String listen_lang = "es"; // en, es, fr, de, it
String speak_lang = "es"; // en, es, fr, de, it

void setup ()
{
  size(1280, 720);
  stroke(255);

  // Init STT automatically starts listening
  stt = new STT(this);
  stt.enableDebug();
  stt.setLanguage(listen_lang); 
  //stt.enableAutoRecord();

  // Some text to display the result
  textFont(createFont("Arial", 24));
  result = "(press any key to record) Then Say something!";

  textAlign(CENTER, CENTER);
  textSize(50);
  
  // Display available Inputs with id and name
  Mixer.Info[] mixerInfo = AudioSystem.getMixerInfo();

  for (int i = 0; i < mixerInfo.length; i++)
  {
    println(i + ": " + mixerInfo[i].getName());
  }  

  // Set input (id: 0)  
  Mixer mixer = AudioSystem.getMixer(mixerInfo[2]);

  // Update the Minim instance that STT is using
  minim = stt.getMinimInstance();
  minim.setInputMixer(mixer);
}

void draw ()
{
  
  background(0);
  //text(result, mouseX, mouseY);

  text(result, 0, 0, width, height);

  if (player != null) {
    translate(0, 250);
    for(int i = 0; i < player.left.size()-1; i++) {
      line(i, 50 + player.left.get(i)*50, i+1, 50 + player.left.get(i+1)*50);
      line(i, 150 + player.right.get(i)*50, i+1, 150 + player.right.get(i+1)*50);
    }
  }

}

// Method is called if transcription was successfull 
void transcribe (String utterance, float confidence) 
{
  println(utterance);
  result = utterance;

  googleTTS(utterance, speak_lang);
  if (player != null) { 
    player.close();
  } // comment this line to layer sounds
  player = minim.loadFile(utterance + ".mp3", 2048);
  player.play();
}

public void keyPressed () {
  stt.begin();
}
public void keyReleased () {
  stt.end();
}

void googleTTS(String txt, String language) {
  String u = "http://translate.google.com/translate_tts?tl=";
  u = u + language + "&q=" + txt;
  u = u.replace(" ", "%20");
  try {
    URL url = new URL(u);
    try {
      URLConnection connection = url.openConnection();
      connection.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.0.3705; .NET CLR 1.1.4322; .NET CLR 1.2.30703)");
      connection.connect();
      InputStream is = connection.getInputStream();
      File f = new File(sketchPath + "/" + txt + ".mp3");
      OutputStream out = new FileOutputStream(f);
      byte buf[] = new byte[1024];
      int len;
      while ((len = is.read(buf)) > 0) {
        out.write(buf, 0, len);
      }
      out.close();
      is.close();
      println("File created: " + txt + ".mp3");
    } catch (IOException e) {
      e.printStackTrace();
    }
  } catch (MalformedURLException e) {
    e.printStackTrace();
  }
}

